{% if errors %} data-toggle="popover" data-container="body" data-trigger="manual"{% if placement %} data-placement="{{ placement }}"{% endif %} data-html="true" data-content="&lt;ul class=&quot;list-unstyled&quot;&gt;{% for error in errors %}&lt;li&gt;{{ error|force_escape }}&lt;/li&gt;{% endfor %}&lt;/ul&gt;"{% endif %}
