.named-multi-widget {padding:0;margin:0;}
.named-multi-widget > li {padding-left:15px;list-style-type:none;margin-top:10px;}
.named-multi-widget > li:first-child {margin-top:0;}
.named-multi-widget > li > label {font-size:90%;}
